#include <stdio.h>

#include "MergeSort.h"

/***********************************************************************
 * main()
 * 
 * Read in a text file, sort the lines, and print the result
 ***********************************************************************/

#define READ_OK		1
#define READ_ERR	0

#define FILEREAD_ERR	-1

/********************************************************
/*
/* ReadLine()
/*     Reads an entire line into a string.
/*	   Returns:  1->OK,  0->Error, EOF (on end of file)
/*
/*******************************************************/

int ReadLine(FILE* inFile, String newString)
{
	int		index;
	int		result;
	char	flushChar;

	index = 0;
	while ((result=fscanf(inFile, "%c", &newString[index]))==1 && newString[index] != '\n' && index < MAXSTRINGLENGTH) {
		index++;
	}

	/* Did we get anything? */

	if (result == EOF) {
		if (index==0) {
			return EOF;
		}
		newString[index] = '\0';
		return READ_OK;
	}

	/*  Now make sure the string is null terminated, and flush anything on the same line still left in the file */

	if (newString[index] != '\n') {
		while ((result = fscanf(inFile, "%c", &flushChar))==1 && flushChar != '\n');
	}
	newString[index] = '\0';
	return READ_OK;

}

/********************************************************
/*
/* ReadFullFile()
/*     Reads an entire (text) file into an ArrayOfStrings
/*	   Returns:  FILEREAD_ERR->Error, otherwise number of lines read
/*
/*******************************************************/

int ReadFullFile(FILE* inFile, ArrayOfStrings targetArray)
{
	int		index;
	int		result;

	result = READ_OK;
	for (index=0; index<MAXNUMLINES && result==READ_OK; index++) {
		result = ReadLine(inFile, targetArray[index]);
	}
	if (result == READ_OK || result == EOF) {
		return index;
	} else {
		/* file too big or basic I/O error */
		return FILEREAD_ERR;
	}
}

/********************************************************
/*
/* Main()
/*		Read the file.
/*		Sort the file.
/*		Print the sorted file.
/*
/*******************************************************/


int main(void) {

	char			fileName[FILENAME_MAX+1];	// symbol from <stdio.h>
	FILE*			inFile;
	int				index;
	int				numLines;

	ArrayOfStrings	fileAsArray;

	printf("Enter filename: ");
	scanf("%s", fileName);				// errors are "basically impossible"

	/*  try to open the file  */

	inFile = fopen(fileName, "r");
	if (inFile==NULL) {
		printf ("Couldn't open file %s\n", fileName);
		return -1;
	}

	/* Now try to read the file */

	numLines = ReadFullFile(inFile, fileAsArray);
	fclose(inFile);
	if (numLines == FILEREAD_ERR) {
		printf ("Some error trying to read file %s\n", fileName);
		return -1;
	}

	/* Okay, sort the file */

	MergeSort(fileAsArray, 0, numLines-1);

	/* Phew, done.  Print the sorted array */

	for (index=0; index<numLines; index++) {
		printf ("%s\n", fileAsArray[index]);
	}

	return 0;
}
